#include <stdlib.h>
#include <assert.h>
#include <xenomai/init.h>
#include <alchemy/task.h>
#include <alchemy/timer.h>
#include <execinfo.h>
#include <time.h>
#include <fcntl.h>
#include <unistd.h>

#include "realtime_wrapper.h"

namespace rtwrapper {

void check_wrapper() {
    assert(TM_INFINITE == TM_INFINITE_W);
    assert(TM_NOW == TM_NOW_W);
    assert(TM_NONBLOCK == TM_NONBLOCK_W);

    assert(T_WARNSW == T_WARNSW_W);
    assert(T_JOINABLE == T_JOINABLE_W);
    assert(sizeof(RT_TASK) == sizeof(RT_TASK_W));
}

bool rt_init_proc() {
    static bool init = false;

    if (init) {
        return true;
    }
    check_wrapper();
    int dummy_argc = 5;
    char dummy_argv0[] = "dummy";
    char dummy_argv1[] = "--session";
    char dummy_argv2[] = "demo";
    char dummy_argv3[] = "--mem-pool-size";
    char dummy_argv4[20] = "256M";
    char *dummy_argv[] = {dummy_argv0, dummy_argv1, dummy_argv2, dummy_argv3, dummy_argv4, NULL};
    char *const* const_dummy_argv = dummy_argv;
    xenomai_init(&dummy_argc, &const_dummy_argv);
    init = true;
    
    return true;
}

bool is_rt_task() {
    return NULL != rt_task_self();
}

RTIME_W rt_timer_read_w(void) {
    return rt_timer_read();
}

int rt_task_create_w(RT_TASK_W *task, const char *name, int stksize, int prio, int mode, int cpus[]) {
    int err = -1;

    err = rt_task_create(task, name, stksize, prio, mode);
    if ( 0!=err ) {
        return err;
    }

    if ( 0<=cpus[0] ) {
        cpu_set_t cpuset;
        CPU_ZERO(&cpuset);
        for (int i = 0; cpus[i] != -1 && i < 100; i++) {
            CPU_SET(cpus[i], &cpuset);
        }
        rt_task_set_affinity_w(task, &cpuset);
    }
    return err;
}

int rt_task_delete_w(RT_TASK_W *task) {
    return rt_task_delete((RT_TASK*)task);
}

int rt_task_set_affinity_w(RT_TASK_W *task, const cpu_set_t *cpus) {
    return rt_task_set_affinity((RT_TASK*)task, cpus);
}

int rt_task_start_w(RT_TASK_W *task, void (*entry)(void *arg), void *arg) {
    return rt_task_start((RT_TASK*)task, entry, arg);
}

int rt_task_join_w(RT_TASK_W *task) {
    return rt_task_join((RT_TASK*)task);
}

int rt_task_sleep_w(RTIME_W delay) {
    if (!is_rt_task()) {
        return 1; 
    }
    return rt_task_sleep(delay);
}

RT_TASK_W *rt_task_self_w(void) {
    return (RT_TASK_W*)rt_task_self();
}

} // namespace
